@@ -35,6 +35,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <inttypes.h>
 
 #include "logging.h"
 #include "upnp_connmgr.h"
@@ -43,24 +44,77 @@
 
 static double buffer_duration = 0.0; /* Buffer disbled by default, see #182 */
 
-static void scan_caps(const GstCaps * caps)
+static void scan_mime_list(void)
 {
-	guint i;
+	GstRegistry* registry = NULL;
 
-	g_return_if_fail(caps != NULL);
+#if (GST_VERSION_MAJOR < 1)
+	registry = gst_registry_get_default();
+#else
+	registry = gst_registry_get();
+#endif
 
-	if (gst_caps_is_any(caps)) {
-		return;
-	}
-	if (gst_caps_is_empty(caps)) {
-		return;
+	// Fetch a list of all element factories
+	GList* features =
+		gst_registry_get_feature_list(registry, GST_TYPE_ELEMENT_FACTORY);
+
+	// Save a copy of the list root so we can properly free later
+	GList* root = features;
+
+	while (features != NULL) {
+		GstPluginFeature* feature = GST_PLUGIN_FEATURE(features->data);
+
+		// Advance list
+		features = g_list_next(features);
+
+		// Better be an element factory
+		assert(GST_IS_ELEMENT_FACTORY(feature));
+
+		GstElementFactory* factory = GST_ELEMENT_FACTORY(feature);
+
+		// Ignore elements without pads
+		if (gst_element_factory_get_num_pad_templates(factory) == 0) continue;
+
+		// Fetch a list of all pads
+		const GList* pads = gst_element_factory_get_static_pad_templates(factory);
+
+		while (pads) {
+			GstStaticPadTemplate* padTemplate = (GstStaticPadTemplate*)pads->data;
+
+			// Advance list
+			pads = g_list_next(pads);
+
+			// Skip pads that aren't sinks
+			if (padTemplate->direction != GST_PAD_SINK) continue;
+
+			// This is literally all known pad presences so it should be OK!
+			assert(padTemplate->presence == GST_PAD_ALWAYS ||
+				   padTemplate->presence == GST_PAD_SOMETIMES ||
+				   padTemplate->presence == GST_PAD_REQUEST);
+
+			GstCaps* capabilities = gst_static_caps_get(&padTemplate->static_caps);
+
+			// Skip capabilities that they tell us nothing
+			if (capabilities == NULL || gst_caps_is_any(capabilities) ||
+				gst_caps_is_empty(capabilities))
+			{
+				gst_caps_unref(capabilities);
+				continue;
+			}
+
+			for (guint i = 0; i < gst_caps_get_size(capabilities); i++) {
+				GstStructure* structure = gst_caps_get_structure(capabilities, i);
+
+				register_mime_type(gst_structure_get_name(structure));
+			}
+
+			gst_caps_unref(capabilities);
+		}
 	}
 
-	for (i = 0; i < gst_caps_get_size(caps); i++) {
-		GstStructure *structure = gst_caps_get_structure(caps, i);
-		const char *mime_type = gst_structure_get_name(structure);
-		register_mime_type(mime_type);
-	}
+	// Free any allocated memory
+	gst_plugin_feature_list_free(root);
+
 	// There seem to be all kinds of mime types out there that start with
 	// "audio/" but are not explicitly supported by gstreamer. Let's just
 	// tell the controller that we can handle everything "audio/*" and hope
@@ -68,85 +122,6 @@ static void scan_caps(const GstCaps * caps)
 	register_mime_type("audio/*");
 }
 
-static void scan_pad_templates_info(GstElement *element,
-				    GstElementFactory *factory)
-{
-	const GList *pads;
-	GstPadTemplate *padtemplate;
-	GstElementClass *class;
-
-	class = GST_ELEMENT_GET_CLASS(element);
-
-	if (!class->numpadtemplates) {
-		return;
-	}
-
-	pads = class->padtemplates;
-	while (pads) {
-		padtemplate = (GstPadTemplate *) (pads->data);
-		//GstPad *pad = (GstPad *) (pads->data);
-		pads = g_list_next(pads);
-
-		if ((padtemplate->direction == GST_PAD_SINK) &&
-		    ((padtemplate->presence == GST_PAD_ALWAYS) ||
-		     (padtemplate->presence == GST_PAD_SOMETIMES) ||
-		     (padtemplate->presence == GST_PAD_REQUEST)) &&
-		    (padtemplate->caps)) {
-			scan_caps(padtemplate->caps);
-		}
-	}
-
-}
-
-
-static void scan_mime_list(void)
-{
-	GstRegistry *registry = NULL;
-	GList *plugins = NULL;
-
-#if (GST_VERSION_MAJOR < 1)
-	registry = gst_registry_get_default();
-	plugins = gst_default_registry_get_plugin_list();
-#else
-	registry = gst_registry_get();
-	plugins = gst_registry_get_plugin_list(registry);
-#endif
-
-	while (plugins) {
-		GList *features;
-		GstPlugin *plugin;
-
-		plugin = (GstPlugin *) (plugins->data);
-		plugins = g_list_next(plugins);
-
-		features =
-			gst_registry_get_feature_list_by_plugin(registry,
-							    gst_plugin_get_name
-							    (plugin));
-
-		while (features) {
-			GstPluginFeature *feature;
-
-			feature = GST_PLUGIN_FEATURE(features->data);
-
-			if (GST_IS_ELEMENT_FACTORY(feature)) {
-				GstElementFactory *factory;
-				GstElement *element;
-				factory = GST_ELEMENT_FACTORY(feature);
-				element =
-				    gst_element_factory_create(factory, NULL);
-				if (element) {
-					scan_pad_templates_info(element,
-								factory);
-				}
-			}
-
-			features = g_list_next(features);
-		}
-	}
-}
-
-
 static GstElement *player_ = NULL;
 static char *gsuri_ = NULL;         // locally strdup()ed
 static char *gs_next_uri_ = NULL;   // locally strdup()ed
@@ -289,7 +264,9 @@ static void MetaModify_add_tag(const GstTagList *list, const gchar *tag,
 static gboolean my_bus_callback(GstBus * bus, GstMessage * msg,
 				gpointer data)
 {
-	//GMainLoop *loop = (GMainLoop *) data;
+	(void)bus;
+	(void)data;
+
 	GstMessageType msgType;
 	const GstObject *msgSrc;
 	const gchar *msgSrcName;
@@ -493,6 +470,9 @@ static int output_gstreamer_set_mute(int m) {
 }
 
 static void prepare_next_stream(GstElement *obj, gpointer userdata) {
+	(void)obj;
+	(void)userdata;
+
 	Log_info("gstreamer", "about-to-finish cb: setting uri %s",
 		 gs_next_uri_);
 	free(gsuri_);
@@ -529,7 +509,7 @@ static int output_gstreamer_init(void)
         if (buffer_duration > 0) {
                 gint64 buffer_duration_ns = round(buffer_duration * 1.0e9);
                 Log_info("gstreamer",
-                         "Setting buffer duration to %ldms",
+                         "Setting buffer duration to %" PRId64 "ms",
                          buffer_duration_ns / 1000000);
                 g_object_set(G_OBJECT(player_),
                              "buffer-duration",
@@ -600,14 +580,16 @@ static int output_gstreamer_init(void)
 struct output_module gstreamer_output = {
         .shortname = "gst",
 	.description = "GStreamer multimedia framework",
-	.init        = output_gstreamer_init,
 	.add_options = output_gstreamer_add_options,
+
+	.init        = output_gstreamer_init,
 	.set_uri     = output_gstreamer_set_uri,
 	.set_next_uri= output_gstreamer_set_next_uri,
 	.play        = output_gstreamer_play,
 	.stop        = output_gstreamer_stop,
 	.pause       = output_gstreamer_pause,
 	.seek        = output_gstreamer_seek,
+
 	.get_position = output_gstreamer_get_position,
 	.get_volume  = output_gstreamer_get_volume,
 	.set_volume  = output_gstreamer_set_volume,
