@@ -38,16 +38,16 @@
 #include <sys/socket.h>
 #include <arpa/inet.h>
 
-#include <upnp/upnp.h>
-#include <upnp/ithread.h>
-#include <upnp/upnptools.h>
+#include <upnp.h>
+#include <ithread.h>
+#include <upnptools.h>
 
 #include "logging.h"
 
 #include "xmlescape.h"
 #include "webserver.h"
 #include "xmldoc.h"
-#include "upnp.h"
+#include "upnp_service.h"
 #include "upnp_device.h"
 #include "variable-container.h"
 
@@ -84,7 +84,7 @@ int upnp_add_response(struct action_event *event,
 		UpnpActionRequest_set_ErrStr(event->request, errorMessage);
 		return -1;
 	}
-	
+
 	UpnpActionRequest_set_ActionResult(event->request, actionResult);
 	return 0;
 }
@@ -123,8 +123,9 @@ void upnp_set_error(struct action_event *event, int error_code,
 	UpnpString *errStr = UpnpString_new();
 	UpnpString_set_String(errStr, buffer);
 	UpnpActionRequest_set_ErrStr(event->request, errStr);
-	Log_error("upnp", "%s: %s\n", __FUNCTION__, 
-			     UpnpActionRequest_get_ErrStr_cstr(event->request));
+	Log_error("upnp", "%s: %s (%d)\n", __FUNCTION__,
+		  UpnpActionRequest_get_ErrStr_cstr(event->request),
+		  error_code);
 }
 
 const char *upnp_get_string(struct action_event *event, const char *key)
@@ -161,7 +162,7 @@ const char *upnp_get_string(struct action_event *event, const char *key)
 }
 
 static int handle_subscription_request(struct upnp_device *priv,
-				       UpnpSubscriptionRequest *sr_event)
+				       const UpnpSubscriptionRequest *sr_event)
 {
 	struct service *srv;
 	int rc;
@@ -358,7 +359,7 @@ static int handle_action_request(struct upnp_device *priv,
 		}
 		IXML_Document *actionResult = UpnpActionRequest_get_ActionResult(ar_event);
 		if (actionResult == NULL) {
-			actionResult = UpnpMakeActionResponse(actionName, 
+			actionResult = UpnpMakeActionResponse(actionName,
 						   event_service->service_type, 0, NULL);
 			UpnpActionRequest_set_ActionResult(event.request, actionResult);
 		}
@@ -394,15 +395,16 @@ static UPNP_CALLBACK(event_handler, EventType, event, userdata)
 	struct upnp_device *priv = (struct upnp_device *) userdata;
 	switch (EventType) {
 	case UPNP_CONTROL_ACTION_REQUEST:
-		handle_action_request(priv, (void*) event);
+		handle_action_request(priv, (UpnpActionRequest*)event);
 		break;
 
 	case UPNP_CONTROL_GET_VAR_REQUEST:
-		handle_var_request(priv, (void*) event);
+		handle_var_request(priv, (UpnpStateVarRequest*)event);
 		break;
 
 	case UPNP_EVENT_SUBSCRIPTION_REQUEST:
-		handle_subscription_request(priv, (void*) event);
+		handle_subscription_request(priv,
+				     (const UpnpSubscriptionRequest*)event);
 		break;
 
 	default:
@@ -414,13 +416,13 @@ static UPNP_CALLBACK(event_handler, EventType, event, userdata)
 
 static gboolean initialize_device(struct upnp_device_descriptor *device_def,
 				  struct upnp_device *result_device,
-				  const char *ip_address,
+				  const char *interface_name,
 				  unsigned short port)
 {
 	int rc;
 	char *buf;
 
-	rc = UpnpInit(ip_address, port);
+	rc = UpnpInit2(interface_name, port);
 	/* There have been situations reported in which UPNP had issues
 	 * initializing right after network came up. #129
 	 */
@@ -428,13 +430,13 @@ static gboolean initialize_device(struct upnp_device_descriptor *device_def,
 	static const int kRetryTimeMs = 1000;
 	while (rc != UPNP_E_SUCCESS && retries_left--) {
 		usleep(kRetryTimeMs * 1000);
-		Log_error("upnp", "UpnpInit(ip=%s, port=%d) Error: %s (%d). Retrying... (%ds)",
-			  ip_address, port, UpnpGetErrorMessage(rc), rc, retries_left);
-		rc = UpnpInit(ip_address, port);
+		Log_error("upnp", "UpnpInit2(interface=%s, port=%d) Error: %s (%d). Retrying... (%ds)",
+			  interface_name, port, UpnpGetErrorMessage(rc), rc, retries_left);
+		rc = UpnpInit2(interface_name, port);
 	}
 	if (UPNP_E_SUCCESS != rc) {
-		Log_error("upnp", "UpnpInit(ip=%s, port=%d) Error: %s (%d). Giving up.",
-			  ip_address, port, UpnpGetErrorMessage(rc), rc);
+		Log_error("upnp", "UpnpInit2(interface=%s, port=%d) Error: %s (%d). Giving up.",
+			  interface_name, port, UpnpGetErrorMessage(rc), rc);
 		return FALSE;
 	}
 	Log_info("upnp", "Registered IP=%s port=%d\n",
@@ -481,7 +483,7 @@ static gboolean initialize_device(struct upnp_device_descriptor *device_def,
 }
 
 struct upnp_device *upnp_device_init(struct upnp_device_descriptor *device_def,
-				     const char *ip_address,
+				     const char *interface_name,
 				     unsigned short port)
 {
 	int rc;
@@ -498,7 +500,7 @@ struct upnp_device *upnp_device_init(struct upnp_device_descriptor *device_def,
 		}
 	}
 
-	struct upnp_device *result_device = malloc(sizeof(*result_device));
+	struct upnp_device *result_device = (struct upnp_device*)malloc(sizeof(*result_device));
 	result_device->upnp_device_descriptor = device_def;
 	ithread_mutex_init(&(result_device->device_mutex), NULL);
 
@@ -514,7 +516,7 @@ struct upnp_device *upnp_device_init(struct upnp_device_descriptor *device_def,
 		webserver_register_buf(srv->scpd_url, buf, "text/xml");
 	}
 
-	if (!initialize_device(device_def, result_device, ip_address, port)) {
+	if (!initialize_device(device_def, result_device, interface_name, port)) {
 		UpnpFinish();
 		free(result_device);
 		return NULL;
